/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.beans.session;

import java.util.Map;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;

import kieker.webgui.beans.application.ThemeSwitcherBean;

/**
 * This bean can be used for a single session of a user and stores the currently used theme (look and feel) for this user. Currently the default value being used is
 * the "glass-x"-theme, if none other value can be find within the
 * parameters of the faces context or in the cookies of the user.
 * 
 * @author Nils Christian Ehmke
 * 
 * @see ThemeSwitcherBean
 * @version 1.0
 */
@ManagedBean
@SessionScoped
public final class CurrentThemeBean {

	/**
	 * The default theme used for all users.
	 */
	private static final String DEFAULT_THEME = "glass-x";
	/**
	 * The key to identify the theme in the cookie.
	 */
	private static final String KEY_COOKIE_THEME = "theme";
	/**
	 * The key to identify the theme in the properties.
	 */
	private static final String KEY_THEME = "theme";
	/**
	 * The current theme.
	 */
	private String theme;

	/**
	 * Creates a new instance of this class.
	 */
	public CurrentThemeBean() {
		// Get the parameters within the current context.
		final Map<String, String> params = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
		// Try to find the default theme within the parameters.
		if (params.containsKey(CurrentThemeBean.KEY_THEME)) {
			this.theme = params.get(CurrentThemeBean.KEY_THEME);
		} else {
			// Use the default theme.
			this.theme = CurrentThemeBean.DEFAULT_THEME;
		}

		// Try to find the cookie for the theme.
		final Map<String, Object> cookies = FacesContext.getCurrentInstance().getExternalContext().getRequestCookieMap();

		if (cookies.containsKey(CurrentThemeBean.KEY_COOKIE_THEME)) {
			this.theme = ((Cookie) cookies.get(CurrentThemeBean.KEY_COOKIE_THEME)).getValue();
		}
	}

	/**
	 * Delivers the current theme.
	 * 
	 * @return The currently used theme.
	 */
	public final String getTheme() {
		return this.theme;
	}

	/**
	 * Sets the value of this bean.
	 * 
	 * @param theme
	 *            The new theme to be stored within this instance.
	 */
	public final void setTheme(final String theme) {
		this.theme = theme;

		// Set the theme cookie.
		final Cookie cookie = new Cookie(CurrentThemeBean.KEY_COOKIE_THEME, theme);
		// Try to save it for a year (maximum age)
		cookie.setMaxAge(60 * 60 * 24 * 365);

		// Deliver the cookie
		final HttpServletResponse response = (HttpServletResponse) FacesContext.getCurrentInstance().getExternalContext().getResponse();
		response.addCookie(cookie);
	}
}
